"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AskarWallet = void 0;
const core_1 = require("@aries-framework/core");
const aries_askar_shared_1 = require("@hyperledger/aries-askar-shared");
// eslint-disable-next-line import/order
const bn_js_1 = __importDefault(require("bn.js"));
const isError = (error) => error instanceof Error;
const tsyringe_1 = require("tsyringe");
const utils_1 = require("../utils");
const JweEnvelope_1 = require("./JweEnvelope");
let AskarWallet = class AskarWallet {
    constructor(logger, fileSystem, signingKeyProviderRegistry) {
        this.logger = logger;
        this.fileSystem = fileSystem;
        this.signingKeyProviderRegistry = signingKeyProviderRegistry;
    }
    get isProvisioned() {
        return this.walletConfig !== undefined;
    }
    get isInitialized() {
        return this._store !== undefined;
    }
    get store() {
        if (!this._store) {
            throw new core_1.AriesFrameworkError('Wallet has not been initialized yet. Make sure to await agent.initialize() before using the agent.');
        }
        return this._store;
    }
    get session() {
        if (!this._session) {
            throw new core_1.AriesFrameworkError('No Wallet Session is opened');
        }
        return this._session;
    }
    /**
     * Dispose method is called when an agent context is disposed.
     */
    async dispose() {
        if (this.isInitialized) {
            await this.close();
        }
    }
    /**
     * @throws {WalletDuplicateError} if the wallet already exists
     * @throws {WalletError} if another error occurs
     */
    async create(walletConfig) {
        await this.createAndOpen(walletConfig);
        await this.close();
    }
    /**
     * @throws {WalletDuplicateError} if the wallet already exists
     * @throws {WalletError} if another error occurs
     */
    async createAndOpen(walletConfig) {
        this.logger.debug(`Creating wallet '${walletConfig.id}`);
        const askarWalletConfig = await this.getAskarWalletConfig(walletConfig);
        try {
            this._store = await aries_askar_shared_1.Store.provision({
                recreate: false,
                uri: askarWalletConfig.uri,
                profile: askarWalletConfig.profile,
                keyMethod: askarWalletConfig.keyMethod,
                passKey: askarWalletConfig.passKey,
            });
            this.walletConfig = walletConfig;
            this._session = await this._store.openSession();
        }
        catch (error) {
            // FIXME: Askar should throw a Duplicate error code, but is currently returning Encryption
            // And if we provide the very same wallet key, it will open it without any error
            if ((0, utils_1.isAskarError)(error) &&
                (error.code === utils_1.AskarErrorCode.Encryption || error.code === utils_1.AskarErrorCode.Duplicate)) {
                const errorMessage = `Wallet '${walletConfig.id}' already exists`;
                this.logger.debug(errorMessage);
                throw new core_1.WalletDuplicateError(errorMessage, {
                    walletType: 'AskarWallet',
                    cause: error,
                });
            }
            const errorMessage = `Error creating wallet '${walletConfig.id}'`;
            this.logger.error(errorMessage, {
                error,
                errorMessage: error.message,
            });
            throw new core_1.WalletError(errorMessage, { cause: error });
        }
        this.logger.debug(`Successfully created wallet '${walletConfig.id}'`);
    }
    /**
     * @throws {WalletNotFoundError} if the wallet does not exist
     * @throws {WalletError} if another error occurs
     */
    async open(walletConfig) {
        await this._open(walletConfig);
    }
    /**
     * @throws {WalletNotFoundError} if the wallet does not exist
     * @throws {WalletError} if another error occurs
     */
    async rotateKey(walletConfig) {
        if (!walletConfig.rekey) {
            throw new core_1.WalletError('Wallet rekey undefined!. Please specify the new wallet key');
        }
        await this._open({
            id: walletConfig.id,
            key: walletConfig.key,
            keyDerivationMethod: walletConfig.keyDerivationMethod,
        }, walletConfig.rekey, walletConfig.rekeyDerivationMethod);
    }
    /**
     * @throws {WalletNotFoundError} if the wallet does not exist
     * @throws {WalletError} if another error occurs
     */
    async _open(walletConfig, rekey, rekeyDerivation) {
        var _a;
        if (this._store) {
            throw new core_1.WalletError('Wallet instance already opened. Close the currently opened wallet before re-opening the wallet');
        }
        const askarWalletConfig = await this.getAskarWalletConfig(walletConfig);
        try {
            this._store = await aries_askar_shared_1.Store.open({
                uri: askarWalletConfig.uri,
                keyMethod: askarWalletConfig.keyMethod,
                passKey: askarWalletConfig.passKey,
            });
            if (rekey) {
                await this._store.rekey({
                    passKey: rekey,
                    keyMethod: (_a = (0, utils_1.keyDerivationMethodToStoreKeyMethod)(rekeyDerivation)) !== null && _a !== void 0 ? _a : aries_askar_shared_1.StoreKeyMethod.Raw,
                });
            }
            this._session = await this._store.openSession();
            this.walletConfig = walletConfig;
        }
        catch (error) {
            if ((0, utils_1.isAskarError)(error) && error.code === utils_1.AskarErrorCode.NotFound) {
                const errorMessage = `Wallet '${walletConfig.id}' not found`;
                this.logger.debug(errorMessage);
                throw new core_1.WalletNotFoundError(errorMessage, {
                    walletType: 'AskarWallet',
                    cause: error,
                });
            }
            else if ((0, utils_1.isAskarError)(error) && error.code === utils_1.AskarErrorCode.Encryption) {
                const errorMessage = `Incorrect key for wallet '${walletConfig.id}'`;
                this.logger.debug(errorMessage);
                throw new core_1.WalletInvalidKeyError(errorMessage, {
                    walletType: 'AskarWallet',
                    cause: error,
                });
            }
            throw new core_1.WalletError(`Error opening wallet ${walletConfig.id}. ERROR CODE ${error.code} MESSAGE ${error.message}`, { cause: error });
        }
        this.logger.debug(`Wallet '${walletConfig.id}' opened with handle '${this._store.handle.handle}'`);
    }
    /**
     * @throws {WalletNotFoundError} if the wallet does not exist
     * @throws {WalletError} if another error occurs
     */
    async delete() {
        if (!this.walletConfig) {
            throw new core_1.WalletError('Can not delete wallet that does not have wallet config set. Make sure to call create wallet before deleting the wallet');
        }
        this.logger.info(`Deleting wallet '${this.walletConfig.id}'`);
        if (this._store) {
            await this.close();
        }
        try {
            const { uri } = (0, utils_1.uriFromWalletConfig)(this.walletConfig, this.fileSystem.dataPath);
            await aries_askar_shared_1.Store.remove(uri);
        }
        catch (error) {
            const errorMessage = `Error deleting wallet '${this.walletConfig.id}': ${error.message}`;
            this.logger.error(errorMessage, {
                error,
                errorMessage: error.message,
            });
            throw new core_1.WalletError(errorMessage, { cause: error });
        }
    }
    async export() {
        // TODO
        throw new core_1.WalletError('AskarWallet Export not yet implemented');
    }
    async import() {
        // TODO
        throw new core_1.WalletError('AskarWallet Import not yet implemented');
    }
    /**
     * @throws {WalletError} if the wallet is already closed or another error occurs
     */
    async close() {
        var _a;
        this.logger.debug(`Closing wallet ${(_a = this.walletConfig) === null || _a === void 0 ? void 0 : _a.id}`);
        if (!this._store) {
            throw new core_1.WalletError('Wallet is in invalid state, you are trying to close wallet that has no handle.');
        }
        try {
            await this.session.close();
            await this.store.close();
            this._session = undefined;
            this._store = undefined;
        }
        catch (error) {
            const errorMessage = `Error closing wallet': ${error.message}`;
            this.logger.error(errorMessage, {
                error,
                errorMessage: error.message,
            });
            throw new core_1.WalletError(errorMessage, { cause: error });
        }
    }
    /**
     * Create a key with an optional seed and keyType.
     * The keypair is also automatically stored in the wallet afterwards
     */
    async createKey({ seed, privateKey, keyType }) {
        try {
            if (seed && privateKey) {
                throw new core_1.WalletError('Only one of seed and privateKey can be set');
            }
            if (seed && !(0, core_1.isValidSeed)(seed, keyType)) {
                throw new core_1.WalletError('Invalid seed provided');
            }
            if (privateKey && !(0, core_1.isValidPrivateKey)(privateKey, keyType)) {
                throw new core_1.WalletError('Invalid private key provided');
            }
            if ((0, utils_1.keyTypeSupportedByAskar)(keyType)) {
                const algorithm = (0, aries_askar_shared_1.keyAlgFromString)(keyType);
                // Create key
                const key = privateKey
                    ? aries_askar_shared_1.Key.fromSecretBytes({ secretKey: privateKey, algorithm })
                    : seed
                        ? aries_askar_shared_1.Key.fromSeed({ seed, algorithm })
                        : aries_askar_shared_1.Key.generate(algorithm);
                // Store key
                try {
                    await this.session.insertKey({ key, name: core_1.TypedArrayEncoder.toBase58(key.publicBytes) });
                    return core_1.Key.fromPublicKey(key.publicBytes, keyType);
                }
                catch (error) {
                    // Handle case where key already exists
                    if ((0, utils_1.isAskarError)(error, utils_1.AskarErrorCode.Duplicate)) {
                        throw new core_1.WalletKeyExistsError('Key already exists');
                    }
                    // Otherwise re-throw error
                    throw error;
                }
            }
            else {
                // Check if there is a signing key provider for the specified key type.
                if (this.signingKeyProviderRegistry.hasProviderForKeyType(keyType)) {
                    const signingKeyProvider = this.signingKeyProviderRegistry.getProviderForKeyType(keyType);
                    const keyPair = await signingKeyProvider.createKeyPair({ seed, privateKey });
                    await this.storeKeyPair(keyPair);
                    return core_1.Key.fromPublicKeyBase58(keyPair.publicKeyBase58, keyType);
                }
                throw new core_1.WalletError(`Unsupported key type: '${keyType}'`);
            }
        }
        catch (error) {
            // If already instance of `WalletError`, re-throw
            if (error instanceof core_1.WalletError)
                throw error;
            if (!isError(error)) {
                throw new core_1.AriesFrameworkError('Attempted to throw error, but it was not of type Error', { cause: error });
            }
            throw new core_1.WalletError(`Error creating key with key type '${keyType}': ${error.message}`, { cause: error });
        }
    }
    /**
     * sign a Buffer with an instance of a Key class
     *
     * @param data Buffer The data that needs to be signed
     * @param key Key The key that is used to sign the data
     *
     * @returns A signature for the data
     */
    async sign({ data, key }) {
        try {
            if ((0, utils_1.keyTypeSupportedByAskar)(key.keyType)) {
                if (!core_1.TypedArrayEncoder.isTypedArray(data)) {
                    throw new core_1.WalletError(`Currently not supporting signing of multiple messages`);
                }
                const keyEntry = await this.session.fetchKey({ name: key.publicKeyBase58 });
                if (!keyEntry) {
                    throw new core_1.WalletError('Key entry not found');
                }
                const signed = keyEntry.key.signMessage({ message: data });
                return core_1.Buffer.from(signed);
            }
            else {
                // Check if there is a signing key provider for the specified key type.
                if (this.signingKeyProviderRegistry.hasProviderForKeyType(key.keyType)) {
                    const signingKeyProvider = this.signingKeyProviderRegistry.getProviderForKeyType(key.keyType);
                    const keyPair = await this.retrieveKeyPair(key.publicKeyBase58);
                    const signed = await signingKeyProvider.sign({
                        data,
                        privateKeyBase58: keyPair.privateKeyBase58,
                        publicKeyBase58: key.publicKeyBase58,
                    });
                    return signed;
                }
                throw new core_1.WalletError(`Unsupported keyType: ${key.keyType}`);
            }
        }
        catch (error) {
            if (!isError(error)) {
                throw new core_1.AriesFrameworkError('Attempted to throw error, but it was not of type Error', { cause: error });
            }
            throw new core_1.WalletError(`Error signing data with verkey ${key.publicKeyBase58}`, { cause: error });
        }
    }
    /**
     * Verify the signature with the data and the used key
     *
     * @param data Buffer The data that has to be confirmed to be signed
     * @param key Key The key that was used in the signing process
     * @param signature Buffer The signature that was created by the signing process
     *
     * @returns A boolean whether the signature was created with the supplied data and key
     *
     * @throws {WalletError} When it could not do the verification
     * @throws {WalletError} When an unsupported keytype is used
     */
    async verify({ data, key, signature }) {
        try {
            if ((0, utils_1.keyTypeSupportedByAskar)(key.keyType)) {
                if (!core_1.TypedArrayEncoder.isTypedArray(data)) {
                    throw new core_1.WalletError(`Currently not supporting verification of multiple messages`);
                }
                const askarKey = aries_askar_shared_1.Key.fromPublicBytes({
                    algorithm: (0, aries_askar_shared_1.keyAlgFromString)(key.keyType),
                    publicKey: key.publicKey,
                });
                return askarKey.verifySignature({ message: data, signature });
            }
            else {
                // Check if there is a signing key provider for the specified key type.
                if (this.signingKeyProviderRegistry.hasProviderForKeyType(key.keyType)) {
                    const signingKeyProvider = this.signingKeyProviderRegistry.getProviderForKeyType(key.keyType);
                    const signed = await signingKeyProvider.verify({
                        data,
                        signature,
                        publicKeyBase58: key.publicKeyBase58,
                    });
                    return signed;
                }
                throw new core_1.WalletError(`Unsupported keyType: ${key.keyType}`);
            }
        }
        catch (error) {
            if (!isError(error)) {
                throw new core_1.AriesFrameworkError('Attempted to throw error, but it was not of type Error', { cause: error });
            }
            throw new core_1.WalletError(`Error verifying signature of data signed with verkey ${key.publicKeyBase58}`, {
                cause: error,
            });
        }
    }
    /**
     * Pack a message using DIDComm V1 algorithm
     *
     * @param payload message to send
     * @param recipientKeys array containing recipient keys in base58
     * @param senderVerkey sender key in base58
     * @returns JWE Envelope to send
     */
    async pack(payload, recipientKeys, senderVerkey // in base58
    ) {
        const cek = aries_askar_shared_1.Key.generate(aries_askar_shared_1.KeyAlgs.Chacha20C20P);
        const senderKey = senderVerkey ? await this.session.fetchKey({ name: senderVerkey }) : undefined;
        const senderExchangeKey = senderKey ? senderKey.key.convertkey({ algorithm: aries_askar_shared_1.KeyAlgs.X25519 }) : undefined;
        const recipients = [];
        for (const recipientKey of recipientKeys) {
            const targetExchangeKey = aries_askar_shared_1.Key.fromPublicBytes({
                publicKey: core_1.Key.fromPublicKeyBase58(recipientKey, core_1.KeyType.Ed25519).publicKey,
                algorithm: aries_askar_shared_1.KeyAlgs.Ed25519,
            }).convertkey({ algorithm: aries_askar_shared_1.KeyAlgs.X25519 });
            if (senderVerkey && senderExchangeKey) {
                const encryptedSender = aries_askar_shared_1.CryptoBox.seal({
                    recipientKey: targetExchangeKey,
                    message: core_1.Buffer.from(senderVerkey),
                });
                const nonce = aries_askar_shared_1.CryptoBox.randomNonce();
                const encryptedCek = aries_askar_shared_1.CryptoBox.cryptoBox({
                    recipientKey: targetExchangeKey,
                    senderKey: senderExchangeKey,
                    message: cek.secretBytes,
                    nonce,
                });
                recipients.push(new JweEnvelope_1.JweRecipient({
                    encryptedKey: encryptedCek,
                    header: {
                        kid: recipientKey,
                        sender: core_1.TypedArrayEncoder.toBase64URL(encryptedSender),
                        iv: core_1.TypedArrayEncoder.toBase64URL(nonce),
                    },
                }));
            }
            else {
                const encryptedCek = aries_askar_shared_1.CryptoBox.seal({
                    recipientKey: targetExchangeKey,
                    message: cek.secretBytes,
                });
                recipients.push(new JweEnvelope_1.JweRecipient({
                    encryptedKey: encryptedCek,
                    header: {
                        kid: recipientKey,
                    },
                }));
            }
        }
        const protectedJson = {
            enc: 'xchacha20poly1305_ietf',
            typ: 'JWM/1.0',
            alg: senderVerkey ? 'Authcrypt' : 'Anoncrypt',
            recipients: recipients.map((item) => core_1.JsonTransformer.toJSON(item)),
        };
        const { ciphertext, tag, nonce } = cek.aeadEncrypt({
            message: core_1.Buffer.from(JSON.stringify(payload)),
            aad: core_1.Buffer.from(core_1.JsonEncoder.toBase64URL(protectedJson)),
        }).parts;
        const envelope = new JweEnvelope_1.JweEnvelope({
            ciphertext: core_1.TypedArrayEncoder.toBase64URL(ciphertext),
            iv: core_1.TypedArrayEncoder.toBase64URL(nonce),
            protected: core_1.JsonEncoder.toBase64URL(protectedJson),
            tag: core_1.TypedArrayEncoder.toBase64URL(tag),
        }).toJson();
        return envelope;
    }
    /**
     * Unpacks a JWE Envelope coded using DIDComm V1 algorithm
     *
     * @param messagePackage JWE Envelope
     * @returns UnpackedMessageContext with plain text message, sender key and recipient key
     */
    async unpack(messagePackage) {
        const protectedJson = core_1.JsonEncoder.fromBase64(messagePackage.protected);
        const alg = protectedJson.alg;
        if (!['Anoncrypt', 'Authcrypt'].includes(alg)) {
            throw new core_1.WalletError(`Unsupported pack algorithm: ${alg}`);
        }
        const recipients = [];
        for (const recip of protectedJson.recipients) {
            const kid = recip.header.kid;
            if (!kid) {
                throw new core_1.WalletError('Blank recipient key');
            }
            const sender = recip.header.sender ? core_1.TypedArrayEncoder.fromBase64(recip.header.sender) : undefined;
            const iv = recip.header.iv ? core_1.TypedArrayEncoder.fromBase64(recip.header.iv) : undefined;
            if (sender && !iv) {
                throw new core_1.WalletError('Missing IV');
            }
            else if (!sender && iv) {
                throw new core_1.WalletError('Unexpected IV');
            }
            recipients.push({
                kid,
                sender,
                iv,
                encrypted_key: core_1.TypedArrayEncoder.fromBase64(recip.encrypted_key),
            });
        }
        let payloadKey, senderKey, recipientKey;
        for (const recipient of recipients) {
            let recipientKeyEntry;
            try {
                recipientKeyEntry = await this.session.fetchKey({ name: recipient.kid });
            }
            catch (error) {
                // TODO: Currently Askar wrapper throws error when key is not found
                // In this case we don't need to throw any error because we should
                // try with other recipient keys
                continue;
            }
            if (recipientKeyEntry) {
                const recip_x = recipientKeyEntry.key.convertkey({ algorithm: aries_askar_shared_1.KeyAlgs.X25519 });
                recipientKey = recipient.kid;
                if (recipient.sender && recipient.iv) {
                    senderKey = core_1.TypedArrayEncoder.toUtf8String(aries_askar_shared_1.CryptoBox.sealOpen({
                        recipientKey: recip_x,
                        ciphertext: recipient.sender,
                    }));
                    const sender_x = aries_askar_shared_1.Key.fromPublicBytes({
                        algorithm: aries_askar_shared_1.KeyAlgs.Ed25519,
                        publicKey: core_1.TypedArrayEncoder.fromBase58(senderKey),
                    }).convertkey({ algorithm: aries_askar_shared_1.KeyAlgs.X25519 });
                    payloadKey = aries_askar_shared_1.CryptoBox.open({
                        recipientKey: recip_x,
                        senderKey: sender_x,
                        message: recipient.encrypted_key,
                        nonce: recipient.iv,
                    });
                }
                else {
                    payloadKey = aries_askar_shared_1.CryptoBox.sealOpen({ ciphertext: recipient.encrypted_key, recipientKey: recip_x });
                }
                break;
            }
        }
        if (!payloadKey) {
            throw new core_1.WalletError('No corresponding recipient key found');
        }
        if (!senderKey && alg === 'Authcrypt') {
            throw new core_1.WalletError('Sender public key not provided for Authcrypt');
        }
        const cek = aries_askar_shared_1.Key.fromSecretBytes({ algorithm: aries_askar_shared_1.KeyAlgs.Chacha20C20P, secretKey: payloadKey });
        const message = cek.aeadDecrypt({
            ciphertext: core_1.TypedArrayEncoder.fromBase64(messagePackage.ciphertext),
            nonce: core_1.TypedArrayEncoder.fromBase64(messagePackage.iv),
            tag: core_1.TypedArrayEncoder.fromBase64(messagePackage.tag),
            aad: core_1.TypedArrayEncoder.fromString(messagePackage.protected),
        });
        return {
            plaintextMessage: core_1.JsonEncoder.fromBuffer(message),
            senderKey,
            recipientKey,
        };
    }
    async generateNonce() {
        try {
            // generate an 80-bit nonce suitable for AnonCreds proofs
            const nonce = aries_askar_shared_1.CryptoBox.randomNonce().slice(0, 10);
            return new bn_js_1.default(nonce).toString();
        }
        catch (error) {
            if (!isError(error)) {
                throw new core_1.AriesFrameworkError('Attempted to throw error, but it was not of type Error', { cause: error });
            }
            throw new core_1.WalletError('Error generating nonce', { cause: error });
        }
    }
    async generateWalletKey() {
        try {
            return aries_askar_shared_1.Store.generateRawKey();
        }
        catch (error) {
            throw new core_1.WalletError('Error generating wallet key', { cause: error });
        }
    }
    async getAskarWalletConfig(walletConfig) {
        var _a;
        const { uri, path } = (0, utils_1.uriFromWalletConfig)(walletConfig, this.fileSystem.dataPath);
        // Make sure path exists before creating the wallet
        if (path) {
            await this.fileSystem.createDirectory(path);
        }
        return {
            uri,
            profile: walletConfig.id,
            // FIXME: Default derivation method should be set somewhere in either agent config or some constants
            keyMethod: (_a = (0, utils_1.keyDerivationMethodToStoreKeyMethod)(walletConfig.keyDerivationMethod)) !== null && _a !== void 0 ? _a : aries_askar_shared_1.StoreKeyMethod.None,
            passKey: walletConfig.key,
        };
    }
    async retrieveKeyPair(publicKeyBase58) {
        try {
            const entryObject = await this.session.fetch({ category: 'KeyPairRecord', name: `key-${publicKeyBase58}` });
            if (entryObject === null || entryObject === void 0 ? void 0 : entryObject.value) {
                return core_1.JsonEncoder.fromString(entryObject === null || entryObject === void 0 ? void 0 : entryObject.value);
            }
            else {
                throw new core_1.WalletError(`No content found for record with public key: ${publicKeyBase58}`);
            }
        }
        catch (error) {
            if ((0, utils_1.isAskarError)(error) &&
                (error.code === utils_1.AskarErrorCode.NotFound ||
                    // FIXME: this is current output from askar wrapper but does not describe specifically a not found scenario
                    error.message === 'Received null pointer. The native library could not find the value.')) {
                throw new core_1.RecordNotFoundError(`KeyPairRecord not found for public key: ${publicKeyBase58}.`, {
                    recordType: 'KeyPairRecord',
                    cause: error,
                });
            }
            throw new core_1.WalletError('Error retrieving KeyPair record', { cause: error });
        }
    }
    async storeKeyPair(keyPair) {
        try {
            await this.session.insert({
                category: 'KeyPairRecord',
                name: `key-${keyPair.publicKeyBase58}`,
                value: JSON.stringify(keyPair),
                tags: {
                    keyType: keyPair.keyType,
                },
            });
        }
        catch (error) {
            if ((0, utils_1.isAskarError)(error, utils_1.AskarErrorCode.Duplicate)) {
                throw new core_1.WalletKeyExistsError('Key already exists');
            }
            throw new core_1.WalletError('Error saving KeyPair record', { cause: error });
        }
    }
};
AskarWallet = __decorate([
    (0, tsyringe_1.injectable)(),
    __param(0, (0, tsyringe_1.inject)(core_1.InjectionSymbols.Logger)),
    __param(1, (0, tsyringe_1.inject)(core_1.InjectionSymbols.FileSystem)),
    __metadata("design:paramtypes", [Object, Object, core_1.SigningProviderRegistry])
], AskarWallet);
exports.AskarWallet = AskarWallet;
//# sourceMappingURL=AskarWallet.js.map